;nyquist plug-in
;version 1
;type process
;name "Panning (orbit)..."
;action "Applying orbit panning..."
;info "by David R. Sky\nReleased under terms of GNU Public License"

;control rate "LFO frequency" real "Hz" 0.2 0.01 20 
;control phase "LFO starting phase" int "0=-90 1=90 degrees" 1 0 1
;control pan-l "Left-most pan position" int "0=left 1=right" 0 0 10
;control pan-r "Right-most pan position" int "0=left 1=right" 10 0 10
;control wetness "tremolo wetness" int "percent" 80 0 100
;control low-f "Lowest lowpass filter cutoff frequency" int "hz" 1000 20 20000
;control depth "Pitch modulation depth" real "depth" 0.2 0 0.5

; make sure all lines which start with a semicolon ';'
; are on one line, not word wrapped

; Orbit Panning by David R. Sky
; added tremolo, lowpass filter with LFO sweep, & pitch modulation
; and added corrected LFO panning
; January 12, 2006
; creates the auditory illusion of hearing 
; your stereo audio in orbit -
; One of the two centermost pan position coincides with
; lowest volume, lowest  cutoff frequency of 
; an LFO-modulated lowpass filter,
; and lowest pitch. 
; Released under terms of the GNU Public License
; http://www.opensource.org/licenses/gpl-license.php

; initializing a few arguments
(setf pan-shift (if (= phase 0) -1 1))
(setf phase (if (= phase 0) -90 90))
(setf pan-l (* 0.1 pan-l)) ; true leftmost pan position
(setf pan-r (* 0.1 pan-r)) ; true rightmost pan position
; highest sweep of lowpass filter cutoff frequency
(setf high-f 20000)
; two variables needed for snd-tapv
(setf maxdepth 0.5)
(setf offset 0.25)

; function to make stereo sound mono, panned to center
(defun mono (sound)
(vector (sum (aref sound 0) (aref sound 1))
(sum (aref sound 1) (aref sound 0))))

; panning function
(defun pan2 (sound where)
   (vector (mult (aref sound 0) (sum 1 (mult -1 where)))
       (mult (aref sound 1) where)))

; LFO function used for panning and the lowpass filter
(defun get-lfo (rate phase lower upper)
; center of LFO sweep
; which without modification is 0
(sum (* 0.5 (+ lower upper))
; factor - how much to multiply sweep of LFO
; which without modification goes from -1 to +1
(mult 0.5 (abs (- upper lower))
; unmodified LFO signal
(lfo rate 1.0 *sine-table* phase))))

; tremolo function, taken from Audacity's tremolo.ny
(defun tremolo (rate phase wetness sound)
(mult (sum (const (- 1.0 (/ wetness 200.0))) (scale (/ wetness 200.0) 
(lfo rate 1.0 *sine-table* phase))) sound))

; function for LFO-controlled pitch change
; for stereo signal
(defun orbit-pitch (depth maxdepth offset phase rate sound)
(vector 
; left channel
(snd-tapv (aref sound 0) offset (mult depth 
(lfo rate 1.0 *sine-table* (+ phase 90))) maxdepth) 
; right channel
(snd-tapv (aref sound 1) offset (mult depth 
(lfo rate 1.0 *sine-table* (+ phase 90))) maxdepth)))

; stereo normalize function,
; without which audio is sometimes clipped in l/r pan positions
(defun normalize (signal)
(setf x (max (peak (aref signal 0) ny:all) (peak (aref signal 1) ny:all)))
(scale (/ 0.95 x) signal))


; applying orbit panning
(normalize 
(orbit-pitch depth maxdepth offset phase rate 
(pan2 
(tremolo rate phase wetness 
(lp (mono s)
; LFO modulated cutoff frequency of lowpass filter
(get-lfo rate phase low-f high-f)
) ; end lp
) ; end tremolo
; LFO modulated argument 'where' for pan function
(get-lfo rate (+ phase (* pan-shift 90)) pan-l pan-r)
) ; end pan2
) ; end orbit-pitch
) ; end normalize

